@@ -106,12 +106,6 @@
 GST_DEBUG_CATEGORY_STATIC (dvdemux_debug);
 #define GST_CAT_DEFAULT dvdemux_debug
 
-static const GstElementDetails dvdemux_details =
-GST_ELEMENT_DETAILS ("DV system stream demuxer",
-    "Codec/Demuxer",
-    "Uses libdv to separate DV audio from DV video (libdv.sourceforge.net)",
-    "Erik Walthinsen <omega@cse.ogi.edu>, Wim Taymans <wim@fluendo.com>");
-
 static GstStaticPadTemplate sink_temp = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -186,7 +180,10 @@ gst_dvdemux_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&audio_src_temp));
 
-  gst_element_class_set_details (element_class, &dvdemux_details);
+  gst_element_class_set_details_simple (element_class,
+      "DV system stream demuxer", "Codec/Demuxer",
+      "Uses libdv to separate DV audio from DV video (libdv.sourceforge.net)",
+      "Erik Walthinsen <omega@cse.ogi.edu>, Wim Taymans <wim@fluendo.com>");
 
   GST_DEBUG_CATEGORY_INIT (dvdemux_debug, "dvdemux", 0, "DV demuxer element");
 }
@@ -200,7 +197,7 @@ gst_dvdemux_class_init (GstDVDemuxClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_dvdemux_finalize);
+  gobject_class->finalize = gst_dvdemux_finalize;
 
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_dvdemux_change_state);
   gstelement_class->send_event = GST_DEBUG_FUNCPTR (gst_dvdemux_send_event);
@@ -287,58 +284,42 @@ gst_dvdemux_reset (GstDVDemux * dvdemux)
   gst_segment_init (&dvdemux->time_segment, GST_FORMAT_TIME);
 }
 
-static void
-gst_dvdemux_add_video_pad (GstDVDemux * dvdemux)
+static GstPad *
+gst_dvdemux_add_pad (GstDVDemux * dvdemux, GstStaticPadTemplate * template)
 {
-  if (dvdemux->videosrcpad)
-    return;
+  gboolean no_more_pads;
+  GstPad *pad;
 
-  dvdemux->videosrcpad =
-      gst_pad_new_from_static_template (&video_src_temp, "video");
-  gst_pad_set_query_function (dvdemux->videosrcpad,
-      GST_DEBUG_FUNCPTR (gst_dvdemux_src_query));
-  gst_pad_set_query_type_function (dvdemux->videosrcpad,
+  pad = gst_pad_new_from_static_template (template, template->name_template);
+
+  gst_pad_set_query_function (pad, GST_DEBUG_FUNCPTR (gst_dvdemux_src_query));
+
+  gst_pad_set_query_type_function (pad,
       GST_DEBUG_FUNCPTR (gst_dvdemux_get_src_query_types));
-  gst_pad_set_event_function (dvdemux->videosrcpad,
+  gst_pad_set_event_function (pad,
       GST_DEBUG_FUNCPTR (gst_dvdemux_handle_src_event));
-  gst_pad_use_fixed_caps (dvdemux->videosrcpad);
-  gst_pad_set_active (dvdemux->videosrcpad, TRUE);
-  gst_element_add_pad (GST_ELEMENT (dvdemux), dvdemux->videosrcpad);
+  gst_pad_use_fixed_caps (pad);
+  gst_pad_set_active (pad, TRUE);
+  gst_element_add_pad (GST_ELEMENT (dvdemux), pad);
 
-  if (dvdemux->videosrcpad && dvdemux->audiosrcpad)
+  no_more_pads =
+      (dvdemux->videosrcpad != NULL && template == &audio_src_temp) ||
+      (dvdemux->audiosrcpad != NULL && template == &video_src_temp);
+
+  if (no_more_pads)
     gst_element_no_more_pads (GST_ELEMENT (dvdemux));
 
-  gst_pad_push_event (dvdemux->videosrcpad, gst_event_new_new_segment (FALSE,
+  gst_pad_push_event (pad, gst_event_new_new_segment (FALSE,
           dvdemux->byte_segment.rate, GST_FORMAT_TIME,
           dvdemux->time_segment.start, dvdemux->time_segment.stop,
           dvdemux->time_segment.start));
-}
 
-static void
-gst_dvdemux_add_audio_pad (GstDVDemux * dvdemux)
-{
-  if (dvdemux->audiosrcpad)
-    return;
+  if (no_more_pads) {
+    gst_element_found_tags (GST_ELEMENT (dvdemux),
+        gst_tag_list_new_full (GST_TAG_CONTAINER_FORMAT, "DV", NULL));
+  }
 
-  dvdemux->audiosrcpad =
-      gst_pad_new_from_static_template (&audio_src_temp, "audio");
-  gst_pad_set_query_function (dvdemux->audiosrcpad,
-      GST_DEBUG_FUNCPTR (gst_dvdemux_src_query));
-  gst_pad_set_query_type_function (dvdemux->audiosrcpad,
-      GST_DEBUG_FUNCPTR (gst_dvdemux_get_src_query_types));
-  gst_pad_set_event_function (dvdemux->audiosrcpad,
-      GST_DEBUG_FUNCPTR (gst_dvdemux_handle_src_event));
-  gst_pad_use_fixed_caps (dvdemux->audiosrcpad);
-  gst_pad_set_active (dvdemux->audiosrcpad, TRUE);
-  gst_element_add_pad (GST_ELEMENT (dvdemux), dvdemux->audiosrcpad);
-
-  if (dvdemux->videosrcpad && dvdemux->audiosrcpad)
-    gst_element_no_more_pads (GST_ELEMENT (dvdemux));
-
-  gst_pad_push_event (dvdemux->audiosrcpad, gst_event_new_new_segment (FALSE,
-          dvdemux->byte_segment.rate, GST_FORMAT_TIME,
-          dvdemux->time_segment.start, dvdemux->time_segment.stop,
-          dvdemux->time_segment.start));
+  return pad;
 }
 
 static void
@@ -372,7 +353,8 @@ gst_dvdemux_src_convert (GstDVDemux * dvdemux, GstPad * pad,
   if (dvdemux->decoder == NULL)
     goto error;
 
-  GST_INFO_OBJECT (pad, "src_value:%lld, src_format:%d, dest_format:%d",
+  GST_INFO_OBJECT (pad,
+      "src_value:%" G_GINT64_FORMAT ", src_format:%d, dest_format:%d",
       src_value, src_format, *dest_format);
 
   switch (src_format) {
@@ -457,7 +439,8 @@ gst_dvdemux_src_convert (GstDVDemux * dvdemux, GstPad * pad,
   }
 
 done:
-  GST_INFO_OBJECT (pad, "Result : dest_format:%d, dest_value:%lld, res:%d",
+  GST_INFO_OBJECT (pad,
+      "Result : dest_format:%d, dest_value:%" G_GINT64_FORMAT ", res:%d",
       *dest_format, *dest_value, res);
   return res;
 
@@ -476,7 +459,8 @@ gst_dvdemux_sink_convert (GstDVDemux * dvdemux, GstFormat src_format,
   gboolean res = TRUE;
 
   GST_DEBUG_OBJECT (dvdemux, "%d -> %d", src_format, *dest_format);
-  GST_INFO_OBJECT (dvdemux, "src_value:%lld, src_format:%d, dest_format:%d",
+  GST_INFO_OBJECT (dvdemux,
+      "src_value:%" G_GINT64_FORMAT ", src_format:%d, dest_format:%d",
       src_value, src_format, *dest_format);
 
   if (*dest_format == GST_FORMAT_DEFAULT)
@@ -532,7 +516,8 @@ gst_dvdemux_sink_convert (GstDVDemux * dvdemux, GstFormat src_format,
     default:
       res = FALSE;
   }
-  GST_INFO_OBJECT (dvdemux, "Result : dest_format:%d, dest_value:%lld, res:%d",
+  GST_INFO_OBJECT (dvdemux,
+      "Result : dest_format:%d, dest_value:%" G_GINT64_FORMAT ", res:%d",
       *dest_format, *dest_value, res);
 
 done:
@@ -822,7 +807,7 @@ gst_dvdemux_convert_src_pair (GstDVDemux * dvdemux, GstPad * pad,
               src_format, src_start, &dst_format, dst_start))) {
     goto done;
   }
-  GST_INFO ("Finished conversion of start: %lld", *dst_start);
+  GST_INFO ("Finished conversion of start: %" G_GINT64_FORMAT, *dst_start);
 
   GST_INFO ("starting conversion of stop");
   /* bring the format to time on srcpad. */
@@ -831,7 +816,7 @@ gst_dvdemux_convert_src_pair (GstDVDemux * dvdemux, GstPad * pad,
     /* could not convert seek format to time offset */
     goto done;
   }
-  GST_INFO ("Finished conversion of stop: %lld", *dst_stop);
+  GST_INFO ("Finished conversion of stop: %" G_GINT64_FORMAT, *dst_stop);
 done:
   return res;
 }
@@ -850,7 +835,7 @@ gst_dvdemux_convert_sink_pair (GstDVDemux * dvdemux,
               src_format, src_start, &dst_format, dst_start))) {
     goto done;
   }
-  GST_INFO ("Finished conversion of start: %lld", *dst_start);
+  GST_INFO ("Finished conversion of start: %" G_GINT64_FORMAT, *dst_start);
 
   GST_INFO ("starting conversion of stop");
   /* bring the format to time on srcpad. */
@@ -859,7 +844,7 @@ gst_dvdemux_convert_sink_pair (GstDVDemux * dvdemux,
     /* could not convert seek format to time offset */
     goto done;
   }
-  GST_INFO ("Finished conversion of stop: %lld", *dst_stop);
+  GST_INFO ("Finished conversion of stop: %" G_GINT64_FORMAT, *dst_stop);
 done:
   return res;
 }
@@ -1229,12 +1214,38 @@ gst_dvdemux_demux_audio (GstDVDemux * dvdemux, GstBuffer * buffer,
 
   dv_decode_full_audio (dvdemux->decoder, data, dvdemux->audio_buffers);
 
-  if ((num_samples = dv_get_num_samples (dvdemux->decoder)) > 0) {
+  if (G_LIKELY ((num_samples = dv_get_num_samples (dvdemux->decoder)) > 0)) {
     gint16 *a_ptr;
     gint i, j;
     GstBuffer *outbuf;
     gint frequency, channels;
 
+    if (G_UNLIKELY (dvdemux->audiosrcpad == NULL))
+      dvdemux->audiosrcpad = gst_dvdemux_add_pad (dvdemux, &audio_src_temp);
+
+    /* get initial format or check if format changed */
+    frequency = dv_get_frequency (dvdemux->decoder);
+    channels = dv_get_num_channels (dvdemux->decoder);
+
+    if (G_UNLIKELY ((frequency != dvdemux->frequency)
+            || (channels != dvdemux->channels))) {
+      GstCaps *caps;
+
+      dvdemux->frequency = frequency;
+      dvdemux->channels = channels;
+
+      /* and set new caps */
+      caps = gst_caps_new_simple ("audio/x-raw-int",
+          "rate", G_TYPE_INT, frequency,
+          "depth", G_TYPE_INT, 16,
+          "width", G_TYPE_INT, 16,
+          "signed", G_TYPE_BOOLEAN, TRUE,
+          "channels", G_TYPE_INT, channels,
+          "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
+      gst_pad_set_caps (dvdemux->audiosrcpad, caps);
+      gst_caps_unref (caps);
+    }
+
     outbuf = gst_buffer_new_and_alloc (num_samples *
         sizeof (gint16) * dvdemux->channels);
 
@@ -1255,31 +1266,8 @@ gst_dvdemux_demux_audio (GstDVDemux * dvdemux, GstBuffer * buffer,
     dvdemux->audio_offset += num_samples;
     GST_BUFFER_OFFSET_END (outbuf) = dvdemux->audio_offset;
 
-    if (!dvdemux->audiosrcpad)
-      gst_dvdemux_add_audio_pad (dvdemux);
-
-    /* check if format changed */
-    frequency = dv_get_frequency (dvdemux->decoder);
-    channels = dv_get_num_channels (dvdemux->decoder);
-
-    if ((frequency != dvdemux->frequency) || (channels != dvdemux->channels)) {
-      GstCaps *caps;
-
-      dvdemux->frequency = frequency;
-      dvdemux->channels = channels;
-
-      /* and set new caps */
-      caps = gst_caps_new_simple ("audio/x-raw-int",
-          "rate", G_TYPE_INT, frequency,
-          "depth", G_TYPE_INT, 16,
-          "width", G_TYPE_INT, 16,
-          "signed", G_TYPE_BOOLEAN, TRUE,
-          "channels", G_TYPE_INT, channels,
-          "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
-      gst_pad_set_caps (dvdemux->audiosrcpad, caps);
-      gst_caps_unref (caps);
-    }
-
+    if (dvdemux->new_media)
+      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
     gst_buffer_set_caps (outbuf, GST_PAD_CAPS (dvdemux->audiosrcpad));
 
     ret = gst_pad_push (dvdemux->audiosrcpad, outbuf);
@@ -1301,12 +1289,12 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
   gboolean wide;
   GstFlowReturn ret = GST_FLOW_OK;
 
-  if (!dvdemux->videosrcpad)
-    gst_dvdemux_add_video_pad (dvdemux);
+  if (G_UNLIKELY (dvdemux->videosrcpad == NULL))
+    dvdemux->videosrcpad = gst_dvdemux_add_pad (dvdemux, &video_src_temp);
 
   /* get params */
   /* framerate is already up-to-date */
-  height = (dvdemux->PAL ? PAL_HEIGHT : NTSC_HEIGHT);
+  height = dvdemux->decoder->height;
   wide = dv_format_wide (dvdemux->decoder);
 
   /* see if anything changed */
@@ -1317,7 +1305,7 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
     dvdemux->height = height;
     dvdemux->wide = wide;
 
-    if (dvdemux->PAL) {
+    if (dvdemux->decoder->system == e_dv_system_625_50) {
       if (wide) {
         par_x = PAL_WIDE_PAR_X;
         par_y = PAL_WIDE_PAR_Y;
@@ -1355,6 +1343,8 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
   GST_BUFFER_OFFSET_END (outbuf) = dvdemux->video_offset + 1;
   GST_BUFFER_DURATION (outbuf) = duration;
 
+  if (dvdemux->new_media)
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
   gst_buffer_set_caps (outbuf, GST_PAD_CAPS (dvdemux->videosrcpad));
 
   GST_DEBUG ("pushing video %" GST_TIME_FORMAT,
@@ -1367,6 +1357,31 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
   return ret;
 }
 
+static gboolean
+gst_dvdemux_is_new_media (GstDVDemux * dvdemux, GstBuffer * buffer)
+{
+  guint8 *data = GST_BUFFER_DATA (buffer);
+  int aaux_offset;
+  int dif;
+  int n_difs;
+
+  n_difs = dvdemux->decoder->num_dif_seqs;
+
+  for (dif = 0; dif < n_difs; dif++) {
+    if (dif & 1) {
+      aaux_offset = (dif * 12000) + (6 + 16 * 1) * 80 + 3;
+    } else {
+      aaux_offset = (dif * 12000) + (6 + 16 * 4) * 80 + 3;
+    }
+    if (data[aaux_offset + 0] == 0x51) {
+      if ((data[aaux_offset + 2] & 0x80) == 0)
+        return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
 /* takes ownership of buffer */
 static GstFlowReturn
 gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
@@ -1421,8 +1436,13 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
   data = GST_BUFFER_DATA (buffer);
 
   dv_parse_packs (dvdemux->decoder, data);
-  if (G_UNLIKELY (dv_is_new_recording (dvdemux->decoder, data)))
+  dvdemux->new_media = FALSE;
+  if (gst_dvdemux_is_new_media (dvdemux, buffer) &&
+      dvdemux->frames_since_new_media > 2) {
     dvdemux->new_media = TRUE;
+    dvdemux->frames_since_new_media = 0;
+  }
+  dvdemux->frames_since_new_media++;
 
   /* does not take ownership of buffer */
   aret = ret = gst_dvdemux_demux_audio (dvdemux, buffer, duration);
@@ -1481,9 +1501,8 @@ gst_dvdemux_flush (GstDVDemux * dvdemux)
       goto parse_header_error;
 
     /* after parsing the header we know the length of the data */
-    dvdemux->PAL = dv_system_50_fields (dvdemux->decoder);
-    length = dvdemux->frame_len = (dvdemux->PAL ? PAL_BUFFER : NTSC_BUFFER);
-    if (dvdemux->PAL) {
+    length = dvdemux->frame_len = dvdemux->decoder->frame_size;
+    if (dvdemux->decoder->system == e_dv_system_625_50) {
       dvdemux->framerate_numerator = PAL_FRAMERATE_NUMERATOR;
       dvdemux->framerate_denominator = PAL_FRAMERATE_DENOMINATOR;
     } else {
@@ -1607,9 +1626,8 @@ gst_dvdemux_loop (GstPad * pad)
       goto parse_header_error;
 
     /* after parsing the header we know the length of the data */
-    dvdemux->PAL = dv_system_50_fields (dvdemux->decoder);
-    dvdemux->frame_len = (dvdemux->PAL ? PAL_BUFFER : NTSC_BUFFER);
-    if (dvdemux->PAL) {
+    dvdemux->frame_len = dvdemux->decoder->frame_size;
+    if (dvdemux->decoder->system == e_dv_system_625_50) {
       dvdemux->framerate_numerator = PAL_FRAMERATE_NUMERATOR;
       dvdemux->framerate_denominator = PAL_FRAMERATE_DENOMINATOR;
     } else {
