@@ -112,11 +112,18 @@ struct _GstPulseRingBuffer
 
   pa_sample_spec sample_spec;
 
+#ifdef HAVE_PULSE_0_9_16
+  void *m_data;
+  size_t m_towrite;
+  size_t m_writable;
+  gint64 m_offset;
+  gint64 m_lastoffset;
+#endif
+
   gboolean corked:1;
   gboolean in_commit:1;
   gboolean paused:1;
 };
-
 struct _GstPulseRingBufferClass
 {
   GstRingBufferClass parent_class;
@@ -180,7 +187,7 @@ gst_pulseringbuffer_class_init (GstPulseRingBufferClass * klass)
 
   ring_parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_pulseringbuffer_finalize);
+  gobject_class->finalize = gst_pulseringbuffer_finalize;
 
   gstringbuffer_class->open_device =
       GST_DEBUG_FUNCPTR (gst_pulseringbuffer_open_device);
@@ -212,7 +219,7 @@ gst_pulseringbuffer_init (GstPulseRingBuffer * pbuf,
   pbuf->context = NULL;
   pbuf->stream = NULL;
 
-#if HAVE_PULSE_0_9_13
+#ifdef HAVE_PULSE_0_9_13
   pa_sample_spec_init (&pbuf->sample_spec);
 #else
   pbuf->sample_spec.format = PA_SAMPLE_INVALID;
@@ -220,6 +227,14 @@ gst_pulseringbuffer_init (GstPulseRingBuffer * pbuf,
   pbuf->sample_spec.channels = 0;
 #endif
 
+#ifdef HAVE_PULSE_0_9_16
+  pbuf->m_data = NULL;
+  pbuf->m_towrite = 0;
+  pbuf->m_writable = 0;
+  pbuf->m_offset = 0;
+  pbuf->m_lastoffset = 0;
+#endif
+
   pbuf->corked = TRUE;
   pbuf->in_commit = FALSE;
   pbuf->paused = FALSE;
@@ -229,6 +244,21 @@ static void
 gst_pulsering_destroy_stream (GstPulseRingBuffer * pbuf)
 {
   if (pbuf->stream) {
+
+#ifdef HAVE_PULSE_0_9_16
+    if (pbuf->m_data) {
+      /* drop shm memory buffer */
+      pa_stream_cancel_write (pbuf->stream);
+
+      /* reset internal variables */
+      pbuf->m_data = NULL;
+      pbuf->m_towrite = 0;
+      pbuf->m_writable = 0;
+      pbuf->m_offset = 0;
+      pbuf->m_lastoffset = 0;
+    }
+#endif
+
     pa_stream_disconnect (pbuf->stream);
 
     /* Make sure we don't get any further callbacks */
@@ -255,7 +285,7 @@ gst_pulsering_destroy_context (GstPulseRingBuffer * pbuf)
 
     /* Make sure we don't get any further callbacks */
     pa_context_set_state_callback (pbuf->context, NULL, NULL);
-#if HAVE_PULSE_0_9_12
+#ifdef HAVE_PULSE_0_9_12
     pa_context_set_subscribe_callback (pbuf->context, NULL, NULL);
 #endif
 
@@ -306,6 +336,11 @@ gst_pulsering_context_state_cb (pa_context * c, void *userdata)
   state = pa_context_get_state (c);
   GST_LOG_OBJECT (psink, "got new context state %d", state);
 
+  /* psink can be null when we are shutting down and the ringbuffer is already
+   * unparented */
+  if (psink == NULL)
+    return;
+
   switch (state) {
     case PA_CONTEXT_READY:
     case PA_CONTEXT_TERMINATED:
@@ -322,7 +357,7 @@ gst_pulsering_context_state_cb (pa_context * c, void *userdata)
   }
 }
 
-#if HAVE_PULSE_0_9_12
+#ifdef HAVE_PULSE_0_9_12
 static void
 gst_pulsering_context_subscribe_cb (pa_context * c,
     pa_subscription_event_type_t t, uint32_t idx, void *userdata)
@@ -384,7 +419,7 @@ gst_pulseringbuffer_open_device (GstRingBuffer * buf)
   /* register some essential callbacks */
   pa_context_set_state_callback (pbuf->context,
       gst_pulsering_context_state_cb, pbuf);
-#if HAVE_PULSE_0_9_12
+#ifdef HAVE_PULSE_0_9_12
   pa_context_set_subscribe_callback (pbuf->context,
       gst_pulsering_context_subscribe_cb, pbuf);
 #endif
@@ -552,7 +587,7 @@ gst_pulsering_stream_latency_cb (pa_stream * s, void *userdata)
     GST_LOG_OBJECT (psink, "latency update (information unknown)");
     return;
   }
-#if HAVE_PULSE_0_9_11
+#ifdef HAVE_PULSE_0_9_11
   sink_usec = info->configured_sink_usec;
 #else
   sink_usec = 0;
@@ -566,7 +601,36 @@ gst_pulsering_stream_latency_cb (pa_stream * s, void *userdata)
       info->sink_usec, sink_usec);
 }
 
-#if HAVE_PULSE_0_9_15
+static void
+gst_pulsering_stream_suspended_cb (pa_stream * p, void *userdata)
+{
+  GstPulseSink *psink;
+  GstPulseRingBuffer *pbuf;
+
+  pbuf = GST_PULSERING_BUFFER_CAST (userdata);
+  psink = GST_PULSESINK_CAST (GST_OBJECT_PARENT (pbuf));
+
+  if (pa_stream_is_suspended (p))
+    GST_DEBUG_OBJECT (psink, "stream suspended");
+  else
+    GST_DEBUG_OBJECT (psink, "stream resumed");
+}
+
+#ifdef HAVE_PULSE_0_9_11
+static void
+gst_pulsering_stream_started_cb (pa_stream * p, void *userdata)
+{
+  GstPulseSink *psink;
+  GstPulseRingBuffer *pbuf;
+
+  pbuf = GST_PULSERING_BUFFER_CAST (userdata);
+  psink = GST_PULSESINK_CAST (GST_OBJECT_PARENT (pbuf));
+
+  GST_DEBUG_OBJECT (psink, "stream started");
+}
+#endif
+
+#ifdef HAVE_PULSE_0_9_15
 static void
 gst_pulsering_stream_event_cb (pa_stream * p, const char *name,
     pa_proplist * pl, void *userdata)
@@ -606,7 +670,7 @@ gst_pulseringbuffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
   const pa_buffer_attr *actual;
   pa_channel_map channel_map;
   pa_operation *o = NULL;
-#if HAVE_PULSE_0_9_20
+#ifdef HAVE_PULSE_0_9_20
   pa_cvolume v;
 #endif
   pa_cvolume *pv = NULL;
@@ -662,7 +726,13 @@ gst_pulseringbuffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
       gst_pulsering_stream_overflow_cb, pbuf);
   pa_stream_set_latency_update_callback (pbuf->stream,
       gst_pulsering_stream_latency_cb, pbuf);
-#if HAVE_PULSE_0_9_15
+  pa_stream_set_suspended_callback (pbuf->stream,
+      gst_pulsering_stream_suspended_cb, pbuf);
+#ifdef HAVE_PULSE_0_9_11
+  pa_stream_set_started_callback (pbuf->stream,
+      gst_pulsering_stream_started_cb, pbuf);
+#endif
+#ifdef HAVE_PULSE_0_9_15
   pa_stream_set_event_callback (pbuf->stream,
       gst_pulsering_stream_event_cb, pbuf);
 #endif
@@ -680,7 +750,7 @@ gst_pulseringbuffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
   GST_INFO_OBJECT (psink, "prebuf:    %d", wanted.prebuf);
   GST_INFO_OBJECT (psink, "minreq:    %d", wanted.minreq);
 
-#if HAVE_PULSE_0_9_20
+#ifdef HAVE_PULSE_0_9_20
   /* configure volume when we changed it, else we leave the default */
   if (psink->volume_set) {
     GST_LOG_OBJECT (psink, "have volume of %f", psink->volume);
@@ -694,12 +764,12 @@ gst_pulseringbuffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
 
   /* construct the flags */
   flags = PA_STREAM_INTERPOLATE_TIMING | PA_STREAM_AUTO_TIMING_UPDATE |
-#if HAVE_PULSE_0_9_11
+#ifdef HAVE_PULSE_0_9_11
       PA_STREAM_ADJUST_LATENCY |
 #endif
       PA_STREAM_START_CORKED;
 
-#if HAVE_PULSE_0_9_12
+#ifdef HAVE_PULSE_0_9_12
   if (psink->mute_set && psink->mute)
     flags |= PA_STREAM_START_MUTED;
 #endif
@@ -985,6 +1055,7 @@ mainloop_leave_defer_cb (pa_mainloop_api * api, void *userdata)
 
   pulsesink->pa_defer_ran = TRUE;
   pa_threaded_mainloop_signal (pulsesink->mainloop, 0);
+  gst_object_unref (pulsesink);
 }
 
 /* stop playback, we flush everything. */
@@ -1031,6 +1102,7 @@ cleanup:
 
   GST_DEBUG_OBJECT (psink, "scheduling stream status");
   psink->pa_defer_ran = FALSE;
+  gst_object_ref (psink);
   pa_mainloop_api_once (pa_threaded_mainloop_get_api (psink->mainloop),
       mainloop_leave_defer_cb, psink);
 
@@ -1194,6 +1266,153 @@ gst_pulseringbuffer_commit (GstRingBuffer * buf, guint64 * sample,
         "need to write %d samples at offset %" G_GINT64_FORMAT, *toprocess,
         offset);
 
+#ifdef HAVE_PULSE_0_9_16
+    if (offset != pbuf->m_lastoffset)
+      GST_LOG_OBJECT (psink, "discontinuity, offset is %" G_GINT64_FORMAT ", "
+          "last offset was %" G_GINT64_FORMAT, offset, pbuf->m_lastoffset);
+
+    towrite = out_samples * bps;
+
+    /* Only ever write segsize bytes at once. This will
+     * also limit the PA shm buffer to segsize
+     */
+    if (towrite > buf->spec.segsize)
+      towrite = buf->spec.segsize;
+
+    if ((pbuf->m_writable < towrite) || (offset != pbuf->m_lastoffset)) {
+      /* if no room left or discontinuity in offset,
+         we need to flush data and get a new buffer */
+
+      /* flush the buffer if possible */
+      if ((pbuf->m_data != NULL) && (pbuf->m_towrite > 0)) {
+
+        GST_LOG_OBJECT (psink,
+            "flushing %u samples at offset %" G_GINT64_FORMAT,
+            (guint) pbuf->m_towrite / bps, pbuf->m_offset);
+
+        if (pa_stream_write (pbuf->stream, (uint8_t *) pbuf->m_data,
+                pbuf->m_towrite, NULL, pbuf->m_offset, PA_SEEK_ABSOLUTE) < 0) {
+          goto write_failed;
+        }
+      }
+      pbuf->m_towrite = 0;
+      pbuf->m_offset = offset;  /* keep track of current offset */
+
+      /* get a buffer to write in for now on */
+      for (;;) {
+        pbuf->m_writable = pa_stream_writable_size (pbuf->stream);
+
+        if (pbuf->m_writable == (size_t) - 1)
+          goto writable_size_failed;
+
+        pbuf->m_writable /= bps;
+        pbuf->m_writable *= bps;        /* handle only complete samples */
+
+        if (pbuf->m_writable >= towrite)
+          break;
+
+        /* see if we need to uncork because we have no free space */
+        if (pbuf->corked) {
+          if (!gst_pulsering_set_corked (pbuf, FALSE, FALSE))
+            goto uncork_failed;
+        }
+
+        /* we can't write a single byte, wait a bit */
+        GST_LOG_OBJECT (psink, "waiting for free space");
+        pa_threaded_mainloop_wait (psink->mainloop);
+
+        if (pbuf->paused)
+          goto was_paused;
+      }
+
+      /* make sure we only buffer up latency-time samples */
+      if (pbuf->m_writable > buf->spec.segsize) {
+        /* limit buffering to latency-time value */
+        pbuf->m_writable = buf->spec.segsize;
+
+        GST_LOG_OBJECT (psink, "Limiting buffering to %" G_GSIZE_FORMAT,
+            pbuf->m_writable);
+      }
+
+      GST_LOG_OBJECT (psink, "requesting %u bytes of shared memory",
+          pbuf->m_writable);
+      if (pa_stream_begin_write (pbuf->stream, &pbuf->m_data,
+              &pbuf->m_writable) < 0) {
+        GST_LOG_OBJECT (psink, "pa_stream_begin_write() failed");
+        goto writable_size_failed;
+      }
+      GST_LOG_OBJECT (psink, "got %u bytes of shared memory", pbuf->m_writable);
+
+      /* Just to make sure that we didn't get more than requested */
+      if (pbuf->m_writable > buf->spec.segsize) {
+        /* limit buffering to latency-time value */
+        pbuf->m_writable = buf->spec.segsize;
+      }
+    }
+
+    if (pbuf->m_writable < towrite)
+      towrite = pbuf->m_writable;
+    avail = towrite / bps;
+
+    GST_LOG_OBJECT (psink, "writing %u samples at offset %" G_GUINT64_FORMAT,
+        (guint) avail, offset);
+
+    if (G_LIKELY (inr == outr && !reverse)) {
+      /* no rate conversion, simply write out the samples */
+      /* copy the data into internal buffer */
+
+      memcpy ((guint8 *) pbuf->m_data + pbuf->m_towrite, data, towrite);
+      pbuf->m_towrite += towrite;
+      pbuf->m_writable -= towrite;
+
+      data += towrite;
+      in_samples -= avail;
+      out_samples -= avail;
+    } else {
+      guint8 *dest, *d, *d_end;
+
+      /* write into the PulseAudio shm buffer */
+      dest = d = (guint8 *) pbuf->m_data + pbuf->m_towrite;
+      d_end = d + towrite;
+
+      if (!reverse) {
+        if (inr >= outr)
+          /* forward speed up */
+          FWD_UP_SAMPLES (data, data_end, d, d_end);
+        else
+          /* forward slow down */
+          FWD_DOWN_SAMPLES (data, data_end, d, d_end);
+      } else {
+        if (inr >= outr)
+          /* reverse speed up */
+          REV_UP_SAMPLES (data, data_end, d, d_end);
+        else
+          /* reverse slow down */
+          REV_DOWN_SAMPLES (data, data_end, d, d_end);
+      }
+      /* see what we have left to write */
+      towrite = (d - dest);
+      pbuf->m_towrite += towrite;
+      pbuf->m_writable -= towrite;
+
+      avail = towrite / bps;
+    }
+
+    /* flush the buffer if it's full */
+    if ((pbuf->m_data != NULL) && (pbuf->m_towrite > 0)
+        && (pbuf->m_writable == 0)) {
+      GST_LOG_OBJECT (psink, "flushing %u samples at offset %" G_GINT64_FORMAT,
+          (guint) pbuf->m_towrite / bps, pbuf->m_offset);
+
+      if (pa_stream_write (pbuf->stream, (uint8_t *) pbuf->m_data,
+              pbuf->m_towrite, NULL, pbuf->m_offset, PA_SEEK_ABSOLUTE) < 0) {
+        goto write_failed;
+      }
+      pbuf->m_towrite = 0;
+      pbuf->m_offset = offset + towrite;        /* keep track of current offset */
+    }
+#else
+
     for (;;) {
       /* FIXME, this is not quite right */
       if ((avail = pa_stream_writable_size (pbuf->stream)) == (size_t) - 1)
@@ -1272,9 +1491,15 @@ gst_pulseringbuffer_commit (GstRingBuffer * buf, guint64 * sample,
 
       avail = towrite / bps;
     }
+#endif /* HAVE_PULSE_0_9_16 */
+
     *sample += avail;
     offset += avail * bps;
 
+#ifdef HAVE_PULSE_0_9_16
+    pbuf->m_lastoffset = offset;
+#endif
+
     /* check if we need to uncork after writing the samples */
     if (pbuf->corked) {
       const pa_timing_info *info;
@@ -1363,6 +1588,9 @@ static void gst_pulsesink_finalize (GObject * object);
 
 static gboolean gst_pulsesink_event (GstBaseSink * sink, GstEvent * event);
 
+static GstStateChangeReturn gst_pulsesink_change_state (GstElement * element,
+    GstStateChange transition);
+
 static void gst_pulsesink_init_interfaces (GType type);
 
 #if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
@@ -1408,7 +1636,7 @@ gst_pulsesink_init_interfaces (GType type)
     NULL,
     NULL,
   };
-#if HAVE_PULSE_0_9_12
+#ifdef HAVE_PULSE_0_9_12
   static const GInterfaceInfo svol_iface_info = {
     NULL, NULL, NULL
   };
@@ -1446,7 +1674,7 @@ gst_pulsesink_base_init (gpointer g_class)
           "width = (int) 32, "
           "depth = (int) 32, "
           "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 32 ];"
-#if HAVE_PULSE_0_9_15
+#ifdef HAVE_PULSE_0_9_15
           "audio/x-raw-int, "
           "endianness = (int) { " ENDIANNESS " }, "
           "signed = (boolean) TRUE, "
@@ -1502,10 +1730,11 @@ gst_pulsesink_class_init (GstPulseSinkClass * klass)
   GstBaseSinkClass *gstbasesink_class = GST_BASE_SINK_CLASS (klass);
   GstBaseSinkClass *bc;
   GstBaseAudioSinkClass *gstaudiosink_class = GST_BASE_AUDIO_SINK_CLASS (klass);
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_pulsesink_finalize);
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_pulsesink_set_property);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_pulsesink_get_property);
+  gobject_class->finalize = gst_pulsesink_finalize;
+  gobject_class->set_property = gst_pulsesink_set_property;
+  gobject_class->get_property = gst_pulsesink_get_property;
 
   gstbasesink_class->event = GST_DEBUG_FUNCPTR (gst_pulsesink_event);
 
@@ -1513,6 +1742,9 @@ gst_pulsesink_class_init (GstPulseSinkClass * klass)
   bc = g_type_class_peek (GST_TYPE_BASE_SINK);
   gstbasesink_class->activate_pull = GST_DEBUG_FUNCPTR (bc->activate_pull);
 
+  gstelement_class->change_state =
+      GST_DEBUG_FUNCPTR (gst_pulsesink_change_state);
+
   gstaudiosink_class->create_ringbuffer =
       GST_DEBUG_FUNCPTR (gst_pulsesink_create_ringbuffer);
 
@@ -1534,7 +1766,7 @@ gst_pulsesink_class_init (GstPulseSinkClass * klass)
           "Human-readable name of the sound device", DEFAULT_DEVICE_NAME,
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
-#if HAVE_PULSE_0_9_12
+#ifdef HAVE_PULSE_0_9_12
   g_object_class_install_property (gobject_class,
       PROP_VOLUME,
       g_param_spec_double ("volume", "Volume",
@@ -1556,7 +1788,7 @@ gst_pulsesink_get_time (GstClock * clock, GstBaseAudioSink * sink)
   GstPulseRingBuffer *pbuf;
   pa_usec_t time;
 
-  if (sink->ringbuffer == NULL || sink->ringbuffer->spec.rate == 0)
+  if (!sink->ringbuffer || !sink->ringbuffer->acquired)
     return GST_CLOCK_TIME_NONE;
 
   pbuf = GST_PULSERING_BUFFER_CAST (sink->ringbuffer);
@@ -1593,8 +1825,6 @@ server_dead:
 static void
 gst_pulsesink_init (GstPulseSink * pulsesink, GstPulseSinkClass * klass)
 {
-  guint res;
-
   pulsesink->server = NULL;
   pulsesink->device = NULL;
   pulsesink->device_description = NULL;
@@ -1613,22 +1843,10 @@ gst_pulsesink_init (GstPulseSink * pulsesink, GstPulseSinkClass * klass)
   GST_DEBUG_OBJECT (pulsesink, "using pulseaudio version %s",
       pulsesink->pa_version);
 
-  pulsesink->mainloop = pa_threaded_mainloop_new ();
-  g_assert (pulsesink->mainloop != NULL);
-  res = pa_threaded_mainloop_start (pulsesink->mainloop);
-  g_assert (res == 0);
-
   /* TRUE for sinks, FALSE for sources */
   pulsesink->probe = gst_pulseprobe_new (G_OBJECT (pulsesink),
       G_OBJECT_GET_CLASS (pulsesink), PROP_DEVICE, pulsesink->device,
       TRUE, FALSE);
-
-  /* override with a custom clock */
-  if (GST_BASE_AUDIO_SINK (pulsesink)->provided_clock)
-    gst_object_unref (GST_BASE_AUDIO_SINK (pulsesink)->provided_clock);
-  GST_BASE_AUDIO_SINK (pulsesink)->provided_clock =
-      gst_audio_clock_new ("GstPulseSinkClock",
-      (GstAudioClockGetTimeFunc) gst_pulsesink_get_time, pulsesink);
 }
 
 static void
@@ -1636,14 +1854,10 @@ gst_pulsesink_finalize (GObject * object)
 {
   GstPulseSink *pulsesink = GST_PULSESINK_CAST (object);
 
-  pa_threaded_mainloop_stop (pulsesink->mainloop);
-
   g_free (pulsesink->server);
   g_free (pulsesink->device);
   g_free (pulsesink->device_description);
 
-  pa_threaded_mainloop_free (pulsesink->mainloop);
-
   if (pulsesink->probe) {
     gst_pulseprobe_free (pulsesink->probe);
     pulsesink->probe = NULL;
@@ -1652,7 +1866,7 @@ gst_pulsesink_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-#if HAVE_PULSE_0_9_12
+#ifdef HAVE_PULSE_0_9_12
 static void
 gst_pulsesink_set_volume (GstPulseSink * psink, gdouble volume)
 {
@@ -1661,6 +1875,9 @@ gst_pulsesink_set_volume (GstPulseSink * psink, gdouble volume)
   GstPulseRingBuffer *pbuf;
   uint32_t idx;
 
+  if (!psink->mainloop)
+    goto no_mainloop;
+
   pa_threaded_mainloop_lock (psink->mainloop);
 
   GST_DEBUG_OBJECT (psink, "setting volume to %f", volume);
@@ -1689,6 +1906,14 @@ unlock:
   return;
 
   /* ERRORS */
+no_mainloop:
+  {
+    psink->volume = volume;
+    psink->volume_set = TRUE;
+
+    GST_DEBUG_OBJECT (psink, "we have no mainloop");
+    return;
+  }
 no_buffer:
   {
     psink->volume = volume;
@@ -1718,6 +1943,9 @@ gst_pulsesink_set_mute (GstPulseSink * psink, gboolean mute)
   GstPulseRingBuffer *pbuf;
   uint32_t idx;
 
+  if (!psink->mainloop)
+    goto no_mainloop;
+
   pa_threaded_mainloop_lock (psink->mainloop);
 
   GST_DEBUG_OBJECT (psink, "setting mute state to %d", mute);
@@ -1744,6 +1972,14 @@ unlock:
   return;
 
   /* ERRORS */
+no_mainloop:
+  {
+    psink->mute = mute;
+    psink->mute_set = TRUE;
+
+    GST_DEBUG_OBJECT (psink, "we have no mainloop");
+    return;
+  }
 no_buffer:
   {
     psink->mute = mute;
@@ -1802,6 +2038,9 @@ gst_pulsesink_get_volume (GstPulseSink * psink)
   gdouble v = DEFAULT_VOLUME;
   uint32_t idx;
 
+  if (!psink->mainloop)
+    goto no_mainloop;
+
   pa_threaded_mainloop_lock (psink->mainloop);
 
   pbuf = GST_PULSERING_BUFFER_CAST (GST_BASE_AUDIO_SINK (psink)->ringbuffer);
@@ -1821,9 +2060,9 @@ gst_pulsesink_get_volume (GstPulseSink * psink)
       goto unlock;
   }
 
+unlock:
   v = psink->volume;
 
-unlock:
   if (o)
     pa_operation_unref (o);
 
@@ -1837,6 +2076,12 @@ unlock:
   return v;
 
   /* ERRORS */
+no_mainloop:
+  {
+    v = psink->volume;
+    GST_DEBUG_OBJECT (psink, "we have no mainloop");
+    return v;
+  }
 no_buffer:
   {
     GST_DEBUG_OBJECT (psink, "we have no ringbuffer");
@@ -1864,7 +2109,11 @@ gst_pulsesink_get_mute (GstPulseSink * psink)
   uint32_t idx;
   gboolean mute = FALSE;
 
+  if (!psink->mainloop)
+    goto no_mainloop;
+
   pa_threaded_mainloop_lock (psink->mainloop);
+  mute = psink->mute;
 
   pbuf = GST_PULSERING_BUFFER_CAST (GST_BASE_AUDIO_SINK (psink)->ringbuffer);
   if (pbuf == NULL || pbuf->stream == NULL)
@@ -1883,8 +2132,6 @@ gst_pulsesink_get_mute (GstPulseSink * psink)
       goto unlock;
   }
 
-  mute = psink->mute;
-
 unlock:
   if (o)
     pa_operation_unref (o);
@@ -1894,6 +2141,12 @@ unlock:
   return mute;
 
   /* ERRORS */
+no_mainloop:
+  {
+    mute = psink->mute;
+    GST_DEBUG_OBJECT (psink, "we have no mainloop");
+    return mute;
+  }
 no_buffer:
   {
     GST_DEBUG_OBJECT (psink, "we have no ringbuffer");
@@ -1946,6 +2199,9 @@ gst_pulsesink_device_description (GstPulseSink * psink)
   pa_operation *o = NULL;
   gchar *t;
 
+  if (!psink->mainloop)
+    goto no_mainloop;
+
   pa_threaded_mainloop_lock (psink->mainloop);
   pbuf = GST_PULSERING_BUFFER_CAST (GST_BASE_AUDIO_SINK (psink)->ringbuffer);
   if (pbuf == NULL || pbuf->stream == NULL)
@@ -1972,6 +2228,11 @@ unlock:
   return t;
 
   /* ERRORS */
+no_mainloop:
+  {
+    GST_DEBUG_OBJECT (psink, "we have no mainloop");
+    return NULL;
+  }
 no_buffer:
   {
     GST_DEBUG_OBJECT (psink, "we have no ringbuffer");
@@ -2003,7 +2264,7 @@ gst_pulsesink_set_property (GObject * object,
       g_free (pulsesink->device);
       pulsesink->device = g_value_dup_string (value);
       break;
-#if HAVE_PULSE_0_9_12
+#ifdef HAVE_PULSE_0_9_12
     case PROP_VOLUME:
       gst_pulsesink_set_volume (pulsesink, g_value_get_double (value));
       break;
@@ -2034,7 +2295,7 @@ gst_pulsesink_get_property (GObject * object,
     case PROP_DEVICE_NAME:
       g_value_take_string (value, gst_pulsesink_device_description (pulsesink));
       break;
-#if HAVE_PULSE_0_9_12
+#ifdef HAVE_PULSE_0_9_12
     case PROP_VOLUME:
       g_value_set_double (value, gst_pulsesink_get_volume (pulsesink));
       break;
@@ -2090,7 +2351,7 @@ name_failed:
   }
 }
 
-#if HAVE_PULSE_0_9_11
+#ifdef HAVE_PULSE_0_9_11
 static void
 gst_pulsesink_change_props (GstPulseSink * psink, GstTagList * l)
 {
@@ -2192,8 +2453,8 @@ gst_pulsesink_event (GstBaseSink * sink, GstEvent * event)
         gst_tag_list_get_string (l, GST_TAG_PERFORMER, &artist);
 
       if (title && artist)
-        t = buf =
-            g_strdup_printf (_("'%s' by '%s'"), g_strstrip (title),
+        /* TRANSLATORS: 'song title' by 'artist name' */
+        t = buf = g_strdup_printf (_("'%s' by '%s'"), g_strstrip (title),
             g_strstrip (artist));
       else if (title)
         t = g_strstrip (title);
@@ -2211,7 +2472,7 @@ gst_pulsesink_event (GstBaseSink * sink, GstEvent * event)
       g_free (description);
       g_free (buf);
 
-#if HAVE_PULSE_0_9_11
+#ifdef HAVE_PULSE_0_9_11
       gst_pulsesink_change_props (pulsesink, l);
 #endif
 
@@ -2223,3 +2484,55 @@ gst_pulsesink_event (GstBaseSink * sink, GstEvent * event)
 
   return GST_BASE_SINK_CLASS (parent_class)->event (sink, event);
 }
+
+static GstStateChangeReturn
+gst_pulsesink_change_state (GstElement * element, GstStateChange transition)
+{
+  GstPulseSink *pulsesink = GST_PULSESINK (element);
+  GstStateChangeReturn ret;
+  guint res;
+
+  switch (transition) {
+    case GST_STATE_CHANGE_NULL_TO_READY:
+      g_assert (pulsesink->mainloop == NULL);
+      pulsesink->mainloop = pa_threaded_mainloop_new ();
+      g_assert (pulsesink->mainloop != NULL);
+      res = pa_threaded_mainloop_start (pulsesink->mainloop);
+      g_assert (res == 0);
+
+      /* override with a custom clock */
+      if (GST_BASE_AUDIO_SINK (pulsesink)->provided_clock)
+        gst_object_unref (GST_BASE_AUDIO_SINK (pulsesink)->provided_clock);
+      GST_BASE_AUDIO_SINK (pulsesink)->provided_clock =
+          gst_audio_clock_new ("GstPulseSinkClock",
+          (GstAudioClockGetTimeFunc) gst_pulsesink_get_time, pulsesink);
+      gst_element_post_message (element,
+          gst_message_new_clock_provide (GST_OBJECT_CAST (element),
+              GST_BASE_AUDIO_SINK (pulsesink)->provided_clock, TRUE));
+      break;
+    default:
+      break;
+  }
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+
+  switch (transition) {
+    case GST_STATE_CHANGE_READY_TO_NULL:
+      gst_element_post_message (element,
+          gst_message_new_clock_provide (GST_OBJECT_CAST (element), NULL,
+              FALSE));
+      if (GST_BASE_AUDIO_SINK (pulsesink)->provided_clock)
+        gst_object_unref (GST_BASE_AUDIO_SINK (pulsesink)->provided_clock);
+      GST_BASE_AUDIO_SINK (pulsesink)->provided_clock = NULL;
+      if (pulsesink->mainloop) {
+        pa_threaded_mainloop_stop (pulsesink->mainloop);
+        pa_threaded_mainloop_free (pulsesink->mainloop);
+        pulsesink->mainloop = NULL;
+      }
+      break;
+    default:
+      break;
+  }
+
+  return ret;
+}
