@@ -108,18 +108,12 @@
 #include <math.h>
 #include <gst/gst.h>
 #include <gst/audio/audio.h>
-/*#include <liboil/liboil.h>*/
 
 #include "gstlevel.h"
 
 GST_DEBUG_CATEGORY_STATIC (level_debug);
 #define GST_CAT_DEFAULT level_debug
 
-static const GstElementDetails level_details = GST_ELEMENT_DETAILS ("Level",
-    "Filter/Analyzer/Audio",
-    "RMS/Peak/Decaying Peak Level messager for audio/raw",
-    "Thomas Vander Stichele <thomas at apestaart dot org>");
-
 static GstStaticPadTemplate sink_template_factory =
     GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -188,7 +182,10 @@ gst_level_base_init (gpointer g_class)
       gst_static_pad_template_get (&sink_template_factory));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_template_factory));
-  gst_element_class_set_details (element_class, &level_details);
+  gst_element_class_set_details_simple (element_class, "Level",
+      "Filter/Analyzer/Audio",
+      "RMS/Peak/Decaying Peak Level messager for audio/raw",
+      "Thomas Vander Stichele <thomas at apestaart dot org>");
 }
 
 static void
@@ -202,7 +199,7 @@ gst_level_class_init (GstLevelClass * klass)
   gobject_class->finalize = gst_level_finalize;
 
   g_object_class_install_property (gobject_class, PROP_SIGNAL_LEVEL,
-      g_param_spec_boolean ("message", "mesage",
+      g_param_spec_boolean ("message", "message",
           "Post a level message for each passed interval",
           TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_SIGNAL_INTERVAL,
