@@ -21,6 +21,7 @@
 # include "config.h"
 #endif
 
+#include <stdio.h>
 #include <gst/check/gstcheck.h>
 #include <gst/audio/multichannel.h>
 
@@ -400,15 +401,14 @@ src_handoff_float32_8ch (GstElement * src, GstBuffer * buf, GstPad * pad,
   GST_BUFFER_OFFSET (buf) = 0;
   GST_BUFFER_TIMESTAMP (buf) = 0;
 
+  GST_BUFFER_CAPS (buf) = caps;
+
   for (i = 0; i < SAMPLES_PER_BUFFER; ++i) {
     for (c = 0; c < NUM_CHANNELS; ++c) {
       *data = (gfloat) ((i * NUM_CHANNELS) + c);
       ++data;
     }
   }
-
-  gst_buffer_set_caps (buf, caps);
-  gst_caps_unref (caps);
 }
 
 static gboolean
@@ -546,6 +546,7 @@ deinterleave_suite (void)
   TCase *tc_chain = tcase_create ("general");
 
   suite_add_tcase (s, tc_chain);
+  tcase_set_timeout (tc_chain, 180);
   tcase_add_test (tc_chain, test_create_and_unref);
   tcase_add_test (tc_chain, test_2_channels);
   tcase_add_test (tc_chain, test_2_channels_1_linked);
